CON
  _CLKMODE      = XTAL1 + PLL16X
  _XINFREQ      = 5_000_000

'========< Precalculated Delay Times >========
  Clock         = 80_000_000
  Sec5          = Clock * 5
  Sec2          = Clock * 2
  Sec1          = Clock
  ms500         = Clock / 2                   
  ms100         = Clock / 10
  ms50          = Clock / 20      
  ms20          = Clock / 50       
  ms10          = Clock / 100                   
  ms5           = Clock / 200                   
  ms1           = Clock / 1000                   
  us500         = Clock / 2000
  us50          = Clock / 200000
  us2           = Clock / 500000
  us1           = Clock / 1000000
'========< Servo Motor Assignments >=========
  MotorLeftPin       = 8            ' Left Drive Wheel
  MotorRightPin      = 9            ' Right Drive Wheel
  MotorPins          = |<MotorLeftPin + |<MotorRightPin      'All Outputs
'========< Digital Encoder Assignments >========
  EncdrRClk          = 12  ' Right Wheel Encoder Clock
  EncdrRDir          = 13  ' Right Wheel Encoder Direction
  EncdrLClk          = 14  ' Left Wheel Encoder Clock
  EncdrLDir          = 15  ' Left Wheel Encoder Direction
  EncdrMode          = %01110           ' Neg Edge detector  (01000=Pos Edge)
' ===== < Speed calculations >====================
' Speed is given in clock ticks to the servo Drive
  Stopped             = 120_000                 ' 1.5 Milli Seconds
  MaxSpeed            = 40_000                  ' 100%  0.5 Mili Seconds
  Spd10               = MaxSpeed * 10/100       ' 10%
  Spd20               = MaxSpeed * 20/100       ' 20%
  Spd30               = MaxSpeed * 30/100       ' 30%
  Spd40               = MaxSpeed * 40/100       ' 40%
  Spd50               = MaxSpeed * 50/100       ' 50%
  Spd60               = MaxSpeed * 60/100       ' 60%
  Spd70               = MaxSpeed * 70/100       ' 70%
  Spd80               = MaxSpeed * 80/100       ' 80%
  Spd90               = MaxSpeed * 90/100       ' 90%
'=============<  Misc Data >=======================
  PstBaud            = 115200
  #0,Left, Right
  #1,Odometer,PrevOdo,PidOut,DeltaCnt,CurSpd,SetSpd,Kp,Ki,Kd,E,prevE,P,I,D,S

Obj
   Srv    : "Servos"
   Pst    : "Parallax Serial Terminal"

VAR
   long LWheel[15],RWheel[15]
   
DAT
  msg1 byte "Propeller BoeBot with PID",0
  Msg2 byte "PulseWidth      Left Cnt    Right Cnt",0
' msgt byte "0..........0..........0..........0..........0..........0..........0",0

Pub main | pw,Line,TestPw
{{ The purpose of this is to characterize the servo speed curves }}
  ifnot Pst.Start(PstBaud)
     repeat
  waitcnt(cnt+sec2)
  Pst.Clear
  Srv.Start(MotorLeftPin, MotorRightPin)
  Srv.SetSpeed(0,0)                     'Speed is given in Clock ticks (40_000)
  EncoderSetup
  ShoTxt(0,5,@msg2)
  Line := 6
  TestPw := MaxSpeed
  repeat pw from -TestPw to TestPw step TestPw/40
     Srv.SetSpeed(pw,pw)                ' Speed is given in Clock Ticks (40_000)
     waitcnt(cnt+Sec1)                  ' Time to allow speed to settle
     EncoderZero                        ' Set counters to zero                               
     waitcnt(cnt+Sec1)                  ' 
     EncoderData                        ' Fetch the counts
     ShoDec(1,  Line,   pw)             ' Print it out
     ShoDec(18, Line,   LWheel[DeltaCnt])   ' Left wheel Encoder ticks per second
     ShoDec(30, Line++, RWheel[DeltaCnt])   ' Right wheel Encoder ticks per second
  Srv.Stop
  repeat

Pri EncoderSetup
'' This sets up both counter control registers as a neg edge detectors
'' then, sets both counters to zero.  
    Frqa := 1          
    Ctra := (EncdrMode << 26) + EncdrLClk
    Phsa := 0
    Frqb := 1      
    Ctrb := (EncdrMode << 26) + EncdrRClk
    Phsb := 0

Pri EncoderZero               ' This clears the wheel data arrays
       LWheel[Odometer] := 0
       RWheel[Odometer] := 0
       LWheel[DeltaCnt] := 0
       RWheel[DeltaCnt] := 0
       LWheel[CurSpd]   := 0
       RWheel[CurSpd]   := 0
       LWheel[PrevOdo]  := 0
       RWheel[PrevOdo]  := 0
       Phsa := 0
       Phsb := 0

Pri EncoderData                  ' This reads the odometers and updates the wheel data array
       LWheel[PrevOdo]  := LWheel[Odometer]
       RWheel[PrevOdo]  := RWheel[Odometer]
       LWheel[Odometer] := Phsa
       RWheel[Odometer] := Phsb
       LWheel[DeltaCnt] := (LWheel[Odometer] - LWheel[PrevOdo])
       RWheel[DeltaCnt] := (RWheel[Odometer] - RWheel[PrevOdo])
       LWheel[CurSpd]   := LWheel[DeltaCnt]
       RWheel[CurSpd]   := RWheel[DeltaCnt]

Pri GoPst
   Pst.Start(115200)
   waitcnt(cnt+Sec2)
   Pst.Clear
   ShoTxt(10,3,@msg1)

Pri ShoTxt(x,y,msg)
   Pst.Position(x,y)
   Pst.Str(msg)

Pri ShoDec(x,y,n)
   Pst.Position(x,y)
   Pst.Dec(n)
   